// MorphingTest.java
package morphingtool;

import java.awt.*;
import java.awt.geom.AffineTransform;
import javax.swing.*;

public class MorphingTest {

  private JFrame frameImage1,  frameImage2;
  private MorphingTestContainer testContainer1,  testContainer2;
  private AffineTransform transformationBild1,  transformationBild2;
  private Point positionLinkesAuge1,  positionRechtesAuge1,  positionMund1;
  private Point positionLinkesAuge2,  positionRechtesAuge2,  positionMund2;
  // Bild Konstanten
  private final int BILD_BREITE = 300;
  private final int BILD_HOEHE = 300;
  // Die Koordinaten der Augen- und Mundpunkte werden in den gesichtsDreieck
  // Arrays abgelegt. Da der Ausdruck "gesichtsDreieck1[4]" nicht wirklich
  // aussagekräftig ist, wird "gesichtsDreieck1[MUND_X]" geschrieben, damit klar
  // wird, dass damit die x-Koordinate des Mund Punktes des Bildes 1 gemeint
  // ist. Die Konstante MUND_X trägt daher den Wert 4. Der Typ double[] wurde
  // gewählt, da die affinen Transformationen double Werte verlangen.
  private final int GESICHTS_DREIECK_GROESSE = 6;
  private double[] gesichtsDreieck1 = new double[GESICHTS_DREIECK_GROESSE];
  private double[] gesichtsDreieck2 = new double[GESICHTS_DREIECK_GROESSE];
  private final int LINKES_AUGE_X = 0;
  private final int LINKES_AUGE_Y = 1;
  private final int RECHTES_AUGE_X = 2;
  private final int RECHTES_AUGE_Y = 3;
  private final int MUND_X = 4;
  private final int MUND_Y = 5;

  // Constructor
  public MorphingTest() {
    // Fenster Objekte erzeugen
    frameImage1 = new JFrame("Bild1");
    frameImage2 = new JFrame("Bild2");
    // Ecken der roten Startdreiecke definieren
    positionLinkesAuge1 = new Point(12, 54);
    positionRechtesAuge1 = new Point(220, 94);
    positionMund1 = new Point(116, 250);
    positionLinkesAuge2 = new Point(146, 124);
    positionRechtesAuge2 = new Point(292, 110);
    positionMund2 = new Point(230, 280);
    // Rote Startdreiecke in die Container übertragen
    testContainer1 = new MorphingTestContainer(BILD_BREITE, BILD_HOEHE,
      positionLinkesAuge1, positionRechtesAuge1, positionMund1);
    testContainer2 = new MorphingTestContainer(BILD_BREITE, BILD_HOEHE,
      positionLinkesAuge2, positionRechtesAuge2, positionMund2);
    // Kopiere die Punkte zur weiteren Bearbeitung
    kopiereBilderUndPunkteZurWeiterenBearbeitung(
      positionLinkesAuge1, positionRechtesAuge1, positionMund1,
      positionLinkesAuge2, positionRechtesAuge2, positionMund2);
    // Berechne die affinen Transformationen
    macheDieBilderDeckungsgleich();
    // Übertrage die affinen Transformationen in die Container
    testContainer1.setAffineTransformation(transformationBild1);
    testContainer2.setAffineTransformation(transformationBild2);
    // Zeige die Fenster an
    fensterAnzeigen();
  }

  public void kopiereBilderUndPunkteZurWeiterenBearbeitung(Point positionLinkesAuge1,
    Point positionRechtesAuge1, Point positionMund1, Point positionLinkesAuge2,
    Point positionRechtesAuge2, Point positionMund2) {
    // Fülle Dreieck 1
    gesichtsDreieck1[LINKES_AUGE_X] = positionLinkesAuge1.x;
    gesichtsDreieck1[LINKES_AUGE_Y] = positionLinkesAuge1.y;
    gesichtsDreieck1[RECHTES_AUGE_X] = positionRechtesAuge1.x;
    gesichtsDreieck1[RECHTES_AUGE_Y] = positionRechtesAuge1.y;
    gesichtsDreieck1[MUND_X] = positionMund1.x;
    gesichtsDreieck1[MUND_Y] = positionMund1.y;
    // Fülle Dreieck 2
    gesichtsDreieck2[LINKES_AUGE_X] = positionLinkesAuge2.x;
    gesichtsDreieck2[LINKES_AUGE_Y] = positionLinkesAuge2.y;
    gesichtsDreieck2[RECHTES_AUGE_X] = positionRechtesAuge2.x;
    gesichtsDreieck2[RECHTES_AUGE_Y] = positionRechtesAuge2.y;
    gesichtsDreieck2[MUND_X] = positionMund2.x;
    gesichtsDreieck2[MUND_Y] = positionMund2.y;
  }

  public void macheDieBilderDeckungsgleich() {
    // Lösche allfällige alte Transformationen
    transformationBild1 = new AffineTransform();
    transformationBild2 = new AffineTransform();
    // Führe Transformationen durch
    berechneTransformationen();
  }

  private void berechneTransformationen() {
    // Die finale Transformation setzt sich aus folgenden Teiltransformationen
    // zusammen:
    verschiebeLinkeAugenZumUrsprung();
    dreheRechteAugenAufXAchse();
    legeXAchsenRechteAugenUndYAchsenMundAufeinander();
    legeXAchsenMundAufeinander();
    zentriereGesichterInBildmitte();
  }

  private void verschiebeLinkeAugenZumUrsprung() {
    // Bild 1
    AffineTransform teilTransformation = new AffineTransform();
    teilTransformation.translate(-gesichtsDreieck1[LINKES_AUGE_X],
      -gesichtsDreieck1[LINKES_AUGE_Y]);
    transformationBild1.preConcatenate(teilTransformation);
    teilTransformation.transform(gesichtsDreieck1, 0, gesichtsDreieck1, 0, 3);
    // Bild 2
    teilTransformation = new AffineTransform();
    teilTransformation.translate(-gesichtsDreieck2[LINKES_AUGE_X],
      -gesichtsDreieck2[LINKES_AUGE_Y]);
    transformationBild2.preConcatenate(teilTransformation);
    teilTransformation.transform(gesichtsDreieck2, 0, gesichtsDreieck2, 0, 3);
  }

  private void dreheRechteAugenAufXAchse() {
    // Bild 1
    AffineTransform teilTransformation = new AffineTransform();
    teilTransformation.rotate(-Math.atan((gesichtsDreieck1[RECHTES_AUGE_Y] -
      gesichtsDreieck1[LINKES_AUGE_Y]) / (gesichtsDreieck1[RECHTES_AUGE_X] -
      gesichtsDreieck1[LINKES_AUGE_X])));
    transformationBild1.preConcatenate(teilTransformation);
    teilTransformation.transform(gesichtsDreieck1, 0, gesichtsDreieck1, 0, 3);
    // Bild 2
    teilTransformation = new AffineTransform();
    teilTransformation.rotate(-Math.atan((gesichtsDreieck2[RECHTES_AUGE_Y] -
      gesichtsDreieck2[LINKES_AUGE_Y]) / (gesichtsDreieck2[RECHTES_AUGE_X] -
      gesichtsDreieck2[LINKES_AUGE_X])));
    transformationBild2.preConcatenate(teilTransformation);
    teilTransformation.transform(gesichtsDreieck2, 0, gesichtsDreieck2, 0, 3);
  }

  private void legeXAchsenRechteAugenUndYAchsenMundAufeinander() {
    // Da die Teiltransformationen der einzelnen Bilder voneinander abhängen,
    // dürfen die Punkte erst zum Schluss für den nächsten Schritt 
    // transformiert werden
    // Bild 1
    AffineTransform teilTransformation1 = new AffineTransform();
    teilTransformation1.scale((gesichtsDreieck1[RECHTES_AUGE_X] +
      gesichtsDreieck2[RECHTES_AUGE_X]) / (2.0 *
      gesichtsDreieck1[RECHTES_AUGE_X]), (gesichtsDreieck1[MUND_Y] +
      gesichtsDreieck2[MUND_Y]) / (2.0 * gesichtsDreieck1[MUND_Y]));
    transformationBild1.preConcatenate(teilTransformation1);
    // Bild 2
    AffineTransform teilTransformation2 = new AffineTransform();
    teilTransformation2.scale((gesichtsDreieck2[RECHTES_AUGE_X] +
      gesichtsDreieck1[RECHTES_AUGE_X]) / (2.0 *
      gesichtsDreieck2[RECHTES_AUGE_X]), (gesichtsDreieck2[MUND_Y] +
      gesichtsDreieck1[MUND_Y]) / (2.0 * gesichtsDreieck2[MUND_Y]));
    transformationBild2.preConcatenate(teilTransformation2);
    // Transformiere alle Punkte
    teilTransformation1.transform(gesichtsDreieck1, 0, gesichtsDreieck1, 0, 3);
    teilTransformation2.transform(gesichtsDreieck2, 0, gesichtsDreieck2, 0, 3);
  }

  private void legeXAchsenMundAufeinander() {
    // Da die Teiltransformationen der einzelnen Bilder voneinander abhängen,
    // dürfen die Punkte erst zum Schluss für den nächsten Schritt 
    // transformiert werden
    // Bild 1
    AffineTransform teilTransformation1 = new AffineTransform();
    teilTransformation1.shear((gesichtsDreieck2[MUND_X] -
      gesichtsDreieck1[MUND_X]) / (2.0 * gesichtsDreieck1[MUND_Y]), 0.0);
    transformationBild1.preConcatenate(teilTransformation1);
    // Bild 2
    AffineTransform teilTransformation2 = new AffineTransform();
    teilTransformation2.shear((gesichtsDreieck1[MUND_X] -
      gesichtsDreieck2[MUND_X]) / (2.0 * gesichtsDreieck2[MUND_Y]), 0.0);
    transformationBild2.preConcatenate(teilTransformation2);
    // Transformiere alle Punkte
    teilTransformation1.transform(gesichtsDreieck1, 0, gesichtsDreieck1, 0, 3);
    teilTransformation2.transform(gesichtsDreieck2, 0, gesichtsDreieck2, 0, 3);
  }

  private void zentriereGesichterInBildmitte() {
    // Bild 1 & 2
    AffineTransform teilTransformation = new AffineTransform();
    teilTransformation.translate((BILD_BREITE - (gesichtsDreieck1[RECHTES_AUGE_X] -
      gesichtsDreieck1[LINKES_AUGE_X])) / 2.0, (BILD_HOEHE -
      (gesichtsDreieck1[MUND_Y] - gesichtsDreieck1[LINKES_AUGE_Y])) / 2.0);
    transformationBild1.preConcatenate(teilTransformation);
    transformationBild2.preConcatenate(teilTransformation);
  }

  private void fensterAnzeigen() {
    // Generell alle Fenster schöner darstellen
    JFrame.setDefaultLookAndFeelDecorated(true);
    // Knöpfe zum Schliessen "aktivieren"
    frameImage1.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frameImage2.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    // Grösse der Fenster an die Container Grössen anpassen
    frameImage1.getContentPane().add(testContainer1);
    frameImage2.getContentPane().add(testContainer2);
    frameImage1.pack();
    frameImage2.pack();
    // Fenster nebeneinander legen
    frameImage1.setLocation(0, 0);
    frameImage2.setLocation(320, 0);
    // Alle Fenster anzeigen lassen
    frameImage1.setVisible(true);
    frameImage2.setVisible(true);
  }

  public static void main(String[] args) {
    new MorphingTest();
  }
}

