// MorphingContainer.java
package morphingtool;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.*;

public class MorphingContainer extends ImageContainer {

  private BufferedImage imageBuffer1,  imageBuffer2;
  private AffineTransform transformationBild1,  transformationBild2;

  // Constructor
  public MorphingContainer() {
    // Zeige Startbild der Vaterklasse an
    super();
    // Definition der affinen Testtransformationen - wird nur für diesen
    // Expertenteil verwendet
    transformationBild1 = AffineTransform.getScaleInstance(1.5, 1.5);
    transformationBild2 = AffineTransform.getScaleInstance(1.5, 1.5);
  }

  // "Setter" Methode welche auch gerade das Bild erzeugt
  public void erzeugeMorphingBild(BufferedImage imageBuffer1, BufferedImage imageBuffer2,
    Point positionLinkesAuge1, Point positionRechtesAuge1, Point positionMund1,
    Point positionLinkesAuge2, Point positionRechtesAuge2, Point positionMund2,
    int slideValue1, int slideValue2) {
    kopiereBilderZurWeiterenBearbeitung(imageBuffer1, imageBuffer2);
    wendeTransformationenAn();
    ueberblendeDieBilder(slideValue1, slideValue2);
  }

  public void kopiereBilderZurWeiterenBearbeitung(BufferedImage imageBuffer1,
    BufferedImage imageBuffer2) {
    // Klone Bilder
    this.imageBuffer1 = new BufferedImage(imageBuffer1.getWidth(), imageBuffer1.getHeight(),
      imageBuffer1.getType());
    this.imageBuffer1.setData(imageBuffer1.getData());
    this.imageBuffer2 = new BufferedImage(imageBuffer2.getWidth(), imageBuffer2.getHeight(),
      imageBuffer2.getType());
    this.imageBuffer2.setData(imageBuffer2.getData());
  }

  private void wendeTransformationenAn() {
    // Die Transformation funktioniert nur auf Graphics2D Objekten!
    // Erzeuge dazu zuerst zwei leere ArbeitsImageBuffer der richtigen Grösse
    BufferedImage arbeitsimageBuffer1 = new BufferedImage(imageBuffer1.getWidth(),
      imageBuffer1.getHeight(), imageBuffer1.getType());
    BufferedImage arbeitsimageBuffer2 = new BufferedImage(imageBuffer2.getWidth(),
      imageBuffer2.getHeight(), imageBuffer2.getType());
    // Verknüpfe die ArbeitsImageBuffer mit Graphics2D Objekten
    Graphics2D arbeitsGrafik1 = (Graphics2D) arbeitsimageBuffer1.getGraphics();
    Graphics2D arbeitsGrafik2 = (Graphics2D) arbeitsimageBuffer2.getGraphics();
    // Initialisiere die Graphics2D Objekte mit der gewünschten Transformation
    arbeitsGrafik1.setTransform(transformationBild1);
    arbeitsGrafik2.setTransform(transformationBild2);
    // Die beiden zu transformierenden ImageBuffer werden in die Grafics2D 
    // Objekte eingelesen, die Transformationen wird durchgeführt und die 
    // Resultate werden in die verknüpften ArbeitsImageBuffer geschrieben
    arbeitsGrafik1.drawImage(imageBuffer1, 0, 0, null);
    arbeitsGrafik2.drawImage(imageBuffer2, 0, 0, null);
    // Lösche die Graphics2D Objekte wieder
    arbeitsGrafik1.dispose();
    arbeitsGrafik2.dispose();
    // Kopiere die Resultate der ArbeitsImageBuffer in die ImageBuffer
    imageBuffer1 = new BufferedImage(arbeitsimageBuffer1.getWidth(),
      arbeitsimageBuffer1.getHeight(), arbeitsimageBuffer1.getType());
    imageBuffer1.setData(arbeitsimageBuffer1.getData());
    imageBuffer2 = new BufferedImage(arbeitsimageBuffer2.getWidth(),
      arbeitsimageBuffer2.getHeight(), arbeitsimageBuffer2.getType());
    imageBuffer2.setData(arbeitsimageBuffer2.getData());
  }

  private void ueberblendeDieBilder(int slideValue1, int slideValue2) {
    // Die Bilder in "imageBuffer1" und "imageBuffer2" Bilder werden pixelweise
    // gemäss der Slide Einstellungen in "imageBuffer" der Vaterklasse überblendet.
    // Die Art der Überblendung wurde durch Thomas Jampen inspiriert. 
    Color pixelFarbeImageBuffer1, pixelFarbeImageBuffer2, pixelFarbeImageBuffer;
    int morphedRed, morphedGreen, morphedBlue;
    // Bestimmt die Mitte des Bildes
    double xBildMitte = imageBuffer.getWidth() / 2;
    double yBildMitte = imageBuffer.getHeight() / 2;
    // Die Variable slideValue1 bestimmt den äusseren Kreis der Überblendung
    // Der maximale Radius beträgt die halbe Diagonale des Bildes
    int radiusAeussererBlendkreis =
            (int) (Math.sqrt(BILD_HOEHE * BILD_HOEHE + BILD_BREITE * BILD_BREITE)
            / 2.0 * (slideValue1 / 100.0));
    // Die Variable slideValue2 bestimmt den inneren Kreis der Überblendung
    // Der maximale Radius entspricht dem äusseren Kreis der Überblendung
    int radiusInnererBlendkreis =
            (int) (radiusAeussererBlendkreis * slideValue2 / 100.0);
    // Die for-Schleifen rufen nach einander jeden einzelnen Bildpixel über die
    // Koordinaten xPosition und yPosition auf.
    for (int xPosition = 0; xPosition < BILD_BREITE; xPosition++) {
      for (int yPosition = 0; yPosition < BILD_HOEHE; yPosition++) {
        // Distanz des Bildpixels (xPosition, yPosition) zum Bildmittelpunkt
        double abstandMittelpunkt =
                Point.distance(xPosition, yPosition, xBildMitte, yBildMitte);
        // Legt fest, welches Bild wie stark berücksichtigt wird, 1 bedeutet
        // 100%, 0 entsprechend 0%
        double anteilImageBuffer1 = 1; // dummy Wert
        double anteilImageBuffer2 = 0; // dummy Wert
        // Bildpixel liegt ausserhalb äusseren Kreis der Überblendung => Bild 1
        // Bildpixel liegt innerhalb inneren Kreis der Überblendung => Bild 2
        // Dazwischen, linearer Übergang zwischen Bild 1 und Bild 2
        if (abstandMittelpunkt > radiusAeussererBlendkreis) {
          anteilImageBuffer1 = 1;
        } else if (abstandMittelpunkt < radiusInnererBlendkreis) {
          anteilImageBuffer1 = 0;
        } else {
          anteilImageBuffer1 = (abstandMittelpunkt - radiusInnererBlendkreis)
                  / (radiusAeussererBlendkreis - radiusInnererBlendkreis);
        }
        anteilImageBuffer2 = 1 - anteilImageBuffer1;
        // Pixelweise jeden RGB Farbkanal einzeln überblenden
        pixelFarbeImageBuffer1 = new Color(imageBuffer1.getRGB(xPosition,
                yPosition), true);
        pixelFarbeImageBuffer2 = new Color(imageBuffer2.getRGB(xPosition,
                yPosition), true);
        morphedRed = (int) (anteilImageBuffer1
                * pixelFarbeImageBuffer1.getRed()
                + anteilImageBuffer2
                * pixelFarbeImageBuffer2.getRed());
        // Aufgrund der Rundungsfehler können unzulässige Farbwerte über    255
        // entstehen, diese werden auf den Farbwert 255 gesetzt
        if (morphedRed > 255) {
          morphedRed = 255;
        }
        morphedGreen = (int) (anteilImageBuffer1
                * pixelFarbeImageBuffer1.getGreen()
                + anteilImageBuffer2
                * pixelFarbeImageBuffer2.getGreen());
        if (morphedGreen > 255) {
          morphedGreen = 255;
        }
        morphedBlue = (int) (anteilImageBuffer1
                * pixelFarbeImageBuffer1.getBlue()
                + anteilImageBuffer2
                * pixelFarbeImageBuffer2.getBlue());
        if (morphedBlue > 255) {
          morphedBlue = 255;
        }
        // RGB Farbkanäle wieder zusammensetzen
        pixelFarbeImageBuffer = new Color(morphedRed,
                morphedGreen, morphedBlue);
        imageBuffer.setRGB(xPosition, yPosition,
                pixelFarbeImageBuffer.getRGB());
      }
    }
    repaint();
  }
}
