// ImageContainer.java
package morphingtool;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import javax.imageio.*;
import javax.swing.*;

public class ImageContainer extends JPanel {

  // Instanzvariablen
  private Graphics2D angezeigteGrafik;
  protected BufferedImage imageBuffer;
  private Point positionLinkesAuge,  positionRechtesAuge,  positionMund;
  private boolean linkesAugeIstOk = false;
  private boolean rechtesAugeIstOk = false;
  private boolean mundIstOk = false;
  // Bild Konstanten
  protected final int BILD_BREITE = 300;
  protected final int BILD_HOEHE = 300;
  private final Color START_BILD_FARBE = Color.BLUE;

  // Constructor
  public ImageContainer() {
    // Grösse des ImageContainers festlegen
    setPreferredSize(new Dimension(BILD_BREITE, BILD_HOEHE));
    // Erzeuge einen leeren Image Buffer als Speicher für das einzulesende Bild
    imageBuffer = new BufferedImage(BILD_BREITE, BILD_HOEHE, BufferedImage.TYPE_INT_RGB);
    zeichneStartBild();
  }

  protected void zeichneStartBild() {
    // Im ImageContainer kann der Image Buffer nicht direkt angezeigt werden. Der
    // ImageContainer kann jedoch Graphics2D Objekte direkt anzeigen. Daher wird
    // der Inhalt des Image Buffers in ein Graphics2D Objekt geschrieben.
    angezeigteGrafik = (Graphics2D) imageBuffer.getGraphics();
    // Lege die Zeichnungsfarbe fest
    angezeigteGrafik.setColor(START_BILD_FARBE);
    // Überdecke den durch den Image Buffer definierten Bereich im Graphics2D Objekt
    // mit einem Rechteck der vorher gewählten Farbe
    angezeigteGrafik.fillRect(0, 0, BILD_BREITE, BILD_HOEHE);
    // Forciere Anzeige des Startbildes
    repaint();
  }

  // Bei der Handhabung von "Files" können immer wieder einmal unvorhergesehene
  // Fehler auftreten. Diese werden über "throws IOException" weitergeleitet
  public String ladeBildInImageContainer(File ausgewaehltesBild) throws IOException {
    // Das Bild wird im Image Buffer zwischengespeichert
    imageBuffer = ImageIO.read(ausgewaehltesBild);
    // Bei falschen Bildformat, lösche Augen- und Mundpunkte und zeige Startbild an
    if ((BILD_BREITE != imageBuffer.getWidth()) || (BILD_HOEHE != imageBuffer.getHeight())) {
      loescheAllePunkte();
      zeichneStartBild();
      // Hinweis für den Benutzer
      return "Nur " + BILD_BREITE + " x " + BILD_HOEHE + " Pixel zulässig!";
    }
    // Loesche alle Augen- und Mundpunkte
    loescheAllePunkte();
    // Zeichne das Panel neu, damit man das Bild sehen kann
    repaint();
    // Sage, was der Benutzer als nächstes tun soll
    return "Markiere linkes Auge";
  }

  public boolean allePunkteSindOk() {
    // Wenn der dritte Punkt, der Mundpunkt gesetzt ist, sind alle Punkte gesetzt
    return mundIstOk;
  }

  private void loescheAllePunkte() {
    // Wenn alle Booleans auf false gesetzt sind, so wird kein Punkt gezeichnet
    linkesAugeIstOk = false;
    rechtesAugeIstOk = false;
    mundIstOk = false;
  }

  public String setzePunkteUndKommentare(Point neuePosition) {
    if (linkesAugeIstOk) {
      if (rechtesAugeIstOk) {
        positionMund = neuePosition;
        mundIstOk = true;
        // Die System.out.println() Ausgabe braucht es nur im Expertenteil
        System.out.println("Mundpunkt: " + neuePosition);
        // Sage, was der Benutzer als nächstes tun soll
        return "Verschiebe Punkte";
      } else {
        positionRechtesAuge = neuePosition;
        rechtesAugeIstOk = true;
        // Die System.out.println() Ausgabe braucht es nur im Expertenteil
        System.out.println("Rechtes Auge: " + neuePosition);
        // Sage, was der Benutzer als nächstes tun soll
        return "Markiere Mundmittelpunkt";
      }
    } else {
      positionLinkesAuge = neuePosition;
      linkesAugeIstOk = true;
      // Die System.out.println() Ausgabe braucht es nur im Expertenteil
      System.out.println("Linkes Auge: " + neuePosition);
      // Sage, was der Benutzer als nächstes tun soll
      return "Markiere rechtes Auge";
    }
  }

  public void verschiebePunkt(Point mousePosition) {
    // Verschiebe denjenigen Punkt, der am naechsten bei der mousePosition liegt
    if (mousePosition.distance(positionMund) < mousePosition.distance(positionLinkesAuge)) {
      if (mousePosition.distance(positionMund) < mousePosition.distance(positionRechtesAuge)) {
        positionMund = new Point(mousePosition);
      } else {
        positionRechtesAuge = new Point(mousePosition);
      }
    } else {
      if (mousePosition.distance(positionRechtesAuge) < mousePosition.distance(positionLinkesAuge)) {
        positionRechtesAuge = new Point(mousePosition);
      } else {
        positionLinkesAuge = new Point(mousePosition);
      }
    }
    // Die System.out.println() Ausgabe braucht es nur im Expertenteil
    System.out.println("Neue Position: " + mousePosition);
  }

  // Die Methode paintComponent wird von Java automatisch oder im Programm
  // über die Methode repaint() aufgerufen, um das Objekt grafik auf dem
  // Bildschirm darzustellen.
  public void paintComponent(Graphics grafik) {
    // Zuerst wird die überschriebene original Methode aufgerufen
    super.paintComponent(grafik);
    // Java verwendet den alten Typ Graphics für das grafik Objekt. Um auf mehr
    // Methoden zugreifen zu können, casten wir den Typ auf Graphics2D
    angezeigteGrafik = (Graphics2D) grafik;
    // Hier wird alles Dazustellende hinzugefügt
    // Zeichne den ImageBuffer
    angezeigteGrafik.drawImage(imageBuffer, 0, 0, null);
  }

  public BufferedImage getBufferedImage() {
    return imageBuffer;
  }

  public Point getPositionLinkesAuge() {
    return positionLinkesAuge;
  }

  public Point getPositionRechtesAuge() {
    return positionRechtesAuge;
  }

  public Point getPositionMund() {
    return positionMund;
  }
}

