// ImageContainer.java
package morphingtool;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import javax.imageio.*;
import javax.swing.*;

public class ImageContainer extends JPanel {

  // Instanzvariablen
  private Graphics2D angezeigteGrafik;
  protected BufferedImage imageBuffer;
  // Bild Konstanten
  protected final int BILD_BREITE = 300;
  protected final int BILD_HOEHE = 300;
  private final Color START_BILD_FARBE = Color.BLUE;

  // Constructor
  public ImageContainer() {
    // Grösse des ImageContainers festlegen
    setPreferredSize(new Dimension(BILD_BREITE, BILD_HOEHE));
    // Erzeuge einen leeren Image Buffer als Speicher für das einzulesende Bild
    imageBuffer = new BufferedImage(BILD_BREITE, BILD_HOEHE, BufferedImage.TYPE_INT_RGB);
    zeichneStartBild();
  }

  protected void zeichneStartBild() {
    // Im ImageContainer kann der Image Buffer nicht direkt angezeigt werden. Der
    // ImageContainer kann jedoch Graphics2D Objekte direkt anzeigen. Daher wird
    // der Inhalt des Image Buffers in ein Graphics2D Objekt geschrieben.
    angezeigteGrafik = (Graphics2D) imageBuffer.getGraphics();
    // Lege die Zeichnungsfarbe fest
    angezeigteGrafik.setColor(START_BILD_FARBE);
    // Überdecke den durch den Image Buffer definierten Bereich im Graphics2D Objekt
    // mit einem Rechteck der vorher gewählten Farbe
    angezeigteGrafik.fillRect(0, 0, BILD_BREITE, BILD_HOEHE);
    // Forciere Anzeige des Startbildes
    repaint();
  }

  // Bei der Handhabung von "Files" können immer wieder einmal unvorhergesehene
  // Fehler auftreten. Diese werden über "throws IOException" weitergeleitet
  public String ladeBildInImageContainer(File ausgewaehltesBild) throws IOException {
    // Das Bild wird im Image Buffer zwischengespeichert
    imageBuffer = ImageIO.read(ausgewaehltesBild);
    // Zeichne das Panel neu, damit man das Bild sehen kann
    repaint();
    // Sage, was der Benutzer als nächstes tun soll
    return "Markiere linkes Auge";
  }

  // Die Methode paintComponent wird von Java automatisch oder im Programm
  // über die Methode repaint() aufgerufen, um das Objekt grafik auf dem
  // Bildschirm darzustellen. 
  public void paintComponent(Graphics grafik) {
    // Zuerst wird die überschriebene original Methode aufgerufen
    super.paintComponent(grafik);
    // Java verwendet den alten Typ Graphics für das grafik Objekt. Um auf mehr
    // Methoden zugreifen zu können, casten wir den Typ auf Graphics2D
    angezeigteGrafik = (Graphics2D) grafik;
    // Hier wird alles Dazustellende hinzugefügt
    // Zeichne den ImageBuffer
    angezeigteGrafik.drawImage(imageBuffer, 0, 0, null);
  }
}

